@@ -443,6 +443,38 @@ LIB_EXPORT bool l_str_has_prefix(const char *str, const char *prefix)
 	return !strncmp(str, prefix, prefix_len);
 }
 
+/**
+ * l_strlcpy:
+ * @dst: Destination buffer for string
+ * @src: Source buffer containing null-terminated string to copy
+ * @len: Maximum destination buffer space to use
+ *
+ * Copies a string from the @src buffer to the @dst buffer, using no
+ * more than @len bytes in @dst. @dst is guaranteed to be
+ * null-terminated. The caller can determine if the copy was truncated by
+ * checking if the return value is greater than or equal to @len.
+ *
+ * Returns: The length of the @src string, not including the null
+ * terminator.
+ */
+LIB_EXPORT size_t l_strlcpy(char* dst, const char *src, size_t len)
+{
+	size_t src_len = strlen(src);
+
+	if (len) {
+		if (src_len < len) {
+			len = src_len + 1;
+		} else {
+			len -= 1;
+			dst[len] = '\0';
+		}
+
+		memcpy(dst, src, len);
+	}
+
+	return src_len;
+}
+
 /**
  * l_str_has_suffix:
  * @str: A string to be examined
@@ -622,6 +654,65 @@ LIB_EXPORT void l_util_hexdump_two(bool in, const void *buf1, size_t len1,
 	hexdump(' ', buf2, len2, function, user_data);
 }
 
+LIB_EXPORT void l_util_hexdumpv(bool in, const struct iovec *iov,
+					size_t n_iov,
+					l_util_hexdump_func_t function,
+					void *user_data)
+{
+	static const char hexdigits[] = "0123456789abcdef";
+	char str[68];
+	size_t i;
+	size_t len;
+	size_t c;
+	const uint8_t *buf;
+
+	if (unlikely(!iov || !n_iov))
+		return;
+
+	str[0] = in ? '<' : '>';
+
+	for (i = 0, len = 0; i < n_iov; i++)
+		len += iov[i].iov_len;
+
+	c = 0;
+	buf = iov[0].iov_base;
+
+	for (i = 0; i < len; i++, c++) {
+		if (c == iov[0].iov_len) {
+			c = 0;
+			iov += 1;
+			buf = iov[0].iov_base;
+		}
+
+		str[((i % 16) * 3) + 1] = ' ';
+		str[((i % 16) * 3) + 2] = hexdigits[buf[c] >> 4];
+		str[((i % 16) * 3) + 3] = hexdigits[buf[c] & 0xf];
+		str[(i % 16) + 51] = isprint(buf[c]) ? buf[c] : '.';
+
+		if ((i + 1) % 16 == 0) {
+			str[49] = ' ';
+			str[50] = ' ';
+			str[67] = '\0';
+			function(str, user_data);
+			str[0] = ' ';
+		}
+	}
+
+	if (i % 16 > 0) {
+		size_t j;
+		for (j = (i % 16); j < 16; j++) {
+			str[(j * 3) + 1] = ' ';
+			str[(j * 3) + 2] = ' ';
+			str[(j * 3) + 3] = ' ';
+			str[j + 51] = ' ';
+		}
+		str[49] = ' ';
+		str[50] = ' ';
+		str[67] = '\0';
+		function(str, user_data);
+	}
+}
+
 LIB_EXPORT void l_util_debug(l_util_hexdump_func_t function, void *user_data,
 						const char *format, ...)
 {
